@@ -7,10 +7,13 @@ use GuzzleHttp\Event\ErrorEvent;
 use GuzzleHttp\Message\MessageFactory;
 use GuzzleHttp\Message\Response;
 use GuzzleHttp\Exception\RequestException;
+use GuzzleHttp\Query;
 use GuzzleHttp\Ring\Client\MockHandler;
 use GuzzleHttp\Ring\Future\FutureArray;
 use GuzzleHttp\Subscriber\History;
 use GuzzleHttp\Subscriber\Mock;
+use GuzzleHttp\Url;
+use GuzzleHttp\Utils;
 use React\Promise\Deferred;
 
 /**
@@ -28,12 +31,6 @@ class ClientTest extends \PHPUnit_Framework_TestCase
         };
     }
 
-    public function testProvidesDefaultUserAgent()
-    {
-        $ua = Client::getDefaultUserAgent();
-        $this->assertEquals(1, preg_match('#^Guzzle/.+ curl/.+ PHP/.+$#', $ua));
-    }
-
     public function testUsesDefaultDefaultOptions()
     {
         $client = new Client();
@@ -70,6 +67,14 @@ class ClientTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals('http://foo.com/baz/', $client->getBaseUrl());
     }
 
+    /**
+     * @expectedException \InvalidArgumentException
+     */
+    public function testValidatesUriTemplateValue()
+    {
+        new Client(['base_url' => ['http://foo.com/']]);
+    }
+
     /**
      * @expectedException \Exception
      * @expectedExceptionMessage Foo
@@ -128,7 +133,7 @@ class ClientTest extends \PHPUnit_Framework_TestCase
         $client = new Client(['defaults' => ['allow_redirects' => false]]);
         $this->assertFalse($client->getDefaultOption('allow_redirects'));
         $this->assertEquals(
-            ['User-Agent' => Client::getDefaultUserAgent()],
+            ['User-Agent' => Utils::getDefaultUserAgent()],
             $client->getDefaultOption('headers')
         );
     }
@@ -137,7 +142,7 @@ class ClientTest extends \PHPUnit_Framework_TestCase
     {
         $client = new Client();
         $this->assertEquals(
-            ['User-Agent' => Client::getDefaultUserAgent()],
+            ['User-Agent' => Utils::getDefaultUserAgent()],
             $client->getDefaultOption('headers')
         );
     }
@@ -243,6 +248,15 @@ class ClientTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals('custom', $request->getHeader('Foo'));
     }
 
+    public function testCanOverrideDefaultOptionWithNull()
+    {
+        $client = new Client(['defaults' => ['proxy' => 'invalid!']]);
+        $request = $client->createRequest('GET', 'http://foo.com?a=b', [
+            'proxy' => null
+        ]);
+        $this->assertFalse($request->getConfig()->hasKey('proxy'));
+    }
+
     public function testDoesNotOverwriteExistingUA()
     {
         $client = new Client(['defaults' => [
@@ -272,6 +286,15 @@ class ClientTest extends \PHPUnit_Framework_TestCase
         );
     }
 
+    public function testFalsyPathsAreCombinedWithBaseUrl()
+    {
+        $client = new Client(['base_url' => 'http://www.foo.com/baz?bam=bar']);
+        $this->assertEquals(
+            'http://www.foo.com/0',
+            $client->createRequest('GET', '0')->getUrl()
+        );
+    }
+
     public function testUsesBaseUrlCombinedWithProvidedUrlViaUriTemplate()
     {
         $client = new Client(['base_url' => 'http://www.foo.com/baz?bam=bar']);
@@ -311,6 +334,34 @@ class ClientTest extends \PHPUnit_Framework_TestCase
         );
     }
 
+    /**
+     * Test that base URLs ending with a slash are resolved as per RFC3986.
+     *
+     * @link http://tools.ietf.org/html/rfc3986#section-5.2.3
+     */
+    public function testMultipleSubdirectoryWithSlash()
+    {
+        $client = new Client(['base_url' => 'http://www.foo.com/bar/bam/']);
+        $this->assertEquals(
+          'http://www.foo.com/bar/bam/httpfoo',
+          $client->createRequest('GET', 'httpfoo')->getUrl()
+        );
+    }
+
+    /**
+     * Test that base URLs ending without a slash are resolved as per RFC3986.
+     *
+     * @link http://tools.ietf.org/html/rfc3986#section-5.2.3
+     */
+    public function testMultipleSubdirectoryNoSlash()
+    {
+        $client = new Client(['base_url' => 'http://www.foo.com/bar/bam']);
+        $this->assertEquals(
+          'http://www.foo.com/bar/httpfoo',
+          $client->createRequest('GET', 'httpfoo')->getUrl()
+        );
+    }
+
     public function testClientSendsRequests()
     {
         $mock = new MockHandler(['status' => 200, 'headers' => []]);
@@ -582,4 +633,15 @@ class ClientTest extends \PHPUnit_Framework_TestCase
         $this->assertInstanceOf('GuzzleHttp\Message\FutureResponse', $res);
         $this->assertEquals(200, $res->getStatusCode());
     }
+
+    public function testCanUseUrlWithCustomQuery()
+    {
+        $client = new Client();
+        $url = Url::fromString('http://foo.com/bar');
+        $query = new Query(['baz' => '123%20']);
+        $query->setEncodingType(false);
+        $url->setQuery($query);
+        $r = $client->createRequest('GET', $url);
+        $this->assertEquals('http://foo.com/bar?baz=123%20', $r->getUrl());
+    }
 }
