@@ -144,7 +144,7 @@ class MessageFactoryTest extends \PHPUnit_Framework_TestCase
      */
     public function testValidatesRedirects()
     {
-        (new MessageFactory())->createRequest('GET', '/', ['allow_redirects' => []]);
+        (new MessageFactory())->createRequest('GET', '/', ['allow_redirects' => 'foo']);
     }
 
     public function testCanEnableStrictRedirectsAndSpecifyMax()
@@ -352,7 +352,7 @@ class MessageFactoryTest extends \PHPUnit_Framework_TestCase
         $mock = new Mock([new Response(200)]);
         $client = new Client();
         $client->getEmitter()->attach($mock);
-        $request = $client->get('http://test.com', ['subscribers' => [$mock]]);
+        $client->get('http://test.com', ['subscribers' => [$mock]]);
     }
 
     public function testCanDisableExceptions()
@@ -555,7 +555,7 @@ class MessageFactoryTest extends \PHPUnit_Framework_TestCase
         $client->getEmitter()->attach($history);
         $client->post('http://foo.com', [
             'headers' => ['Content-Type' => 'multipart/form-data'],
-            'body' => ['foo' => 'bar']
+            'body'    => ['foo' => 'bar']
         ]);
         $this->assertContains(
             'multipart/form-data; boundary=',
